/*!@file
** @brief   The handling options for file times.
** @remarks There are various options for handling file times.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
**
** @copyright (C) 2014-2018 Ueta System Design Studio\n
** This file is licensed under the terms of the MIT License.\n
** See <https://opensource.org/licenses/MIT>.
*/
#pragma once

/*!
** @brief   The base class of the handling options for file times.
** @remarks -
*/
template <class T>
class CHandlingOptions : public CMessageMap
{
public:

	int m_iCreationTimeOption;      ///< The handling option for the creation time.
	int m_iLastAccessTimeOption;    ///< The handling option for the access time.
	int m_iLastWriteTimeOption;     ///< The handling option for the modification time.

	CFileTimeEx m_ftCreationTime;   ///< The creation time of a file or a directory.
	CFileTimeEx m_ftLastAccessTime; ///< The access time of a file or a directory.
	CFileTimeEx m_ftLastWriteTime;  ///< The modification time of a file or a directory.
	CFileTimeEx m_ftCurrentTime;    ///< The current time.

public:

	/*!
	** @brief   Initializes a new CHandlingOptions object.
	** @remarks -
	*/
	CHandlingOptions()
	{
		m_iCreationTimeOption   = T::CCreationTimeOptions::DoNotChange;
		m_iLastAccessTimeOption = T::CLastAccessTimeOptions::DoNotChange;
		m_iLastWriteTimeOption  = T::CLastWriteTimeOptions::DoNotChange;
	}

	/*!
	** @brief   Exchanges and validates dialog data.
	** @param   [in] bSaveAndValidate indicates the direction of the data transfer.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL DoDataExchange(BOOL bSaveAndValidate = FALSE)
	{
		T* pT = static_cast<T*>(this);

		BOOL bSuccess = TRUE;

		pT->DDX_Option(IDC_OPTION_CREATIONTIME,   m_iCreationTimeOption,   bSaveAndValidate);
		pT->DDX_Option(IDC_OPTION_LASTACCESSTIME, m_iLastAccessTimeOption, bSaveAndValidate);
		pT->DDX_Option(IDC_OPTION_LASTWRITETIME,  m_iLastWriteTimeOption,  bSaveAndValidate);

		if (!bSaveAndValidate)
		{
			OnOptionSelEndOk(IDC_OPTION_CREATIONTIME);
			OnOptionSelEndOk(IDC_OPTION_LASTACCESSTIME);
			OnOptionSelEndOk(IDC_OPTION_LASTWRITETIME);
		}

		if (!pT->DDX_DateTime(IDC_DATETIME_CREATIONTIME,   m_ftCreationTime,   bSaveAndValidate)
		 || !pT->DDX_DateTime(IDC_DATETIME_LASTACCESSTIME, m_ftLastAccessTime, bSaveAndValidate)
		 || !pT->DDX_DateTime(IDC_DATETIME_LASTWRITETIME,  m_ftLastWriteTime,  bSaveAndValidate))
		{
			bSuccess = FALSE;
		}
		if (!bSaveAndValidate)
		{
			OnDateTimeChange(IDC_DATETIME_CREATIONTIME);
			OnDateTimeChange(IDC_DATETIME_LASTACCESSTIME);
			OnDateTimeChange(IDC_DATETIME_LASTWRITETIME);
		}
		return bSuccess;
	}

	BEGIN_MSG_MAP(CHandlingOptions)
		COMMAND_HANDLER(IDC_OPTION_CREATIONTIME, CBN_SELENDOK, OnOptionSelEndOk)
		COMMAND_HANDLER(IDC_OPTION_LASTACCESSTIME, CBN_SELENDOK, OnOptionSelEndOk)
		COMMAND_HANDLER(IDC_OPTION_LASTWRITETIME, CBN_SELENDOK, OnOptionSelEndOk)
		NOTIFY_HANDLER(IDC_DATETIME_CREATIONTIME, DTN_DATETIMECHANGE, OnDateTimeChange)
		NOTIFY_HANDLER(IDC_DATETIME_LASTACCESSTIME, DTN_DATETIMECHANGE, OnDateTimeChange)
		NOTIFY_HANDLER(IDC_DATETIME_LASTWRITETIME, DTN_DATETIMECHANGE, OnDateTimeChange)
		COMMAND_ID_HANDLER(IDC_CURRENT, OnCurrent)
		COMMAND_ID_HANDLER(IDC_BROWSE, OnBrowse)
	END_MSG_MAP()

	/*!
	** @brief   Substitutes for a CBN_SELENDOK notification from the Combo Box control.
	** @param   [in] nID the identifier of the control sending the message.
	** @returns No return value.
	** @remarks -
	*/
	void OnOptionSelEndOk(int nID)
	{
		T* pT = static_cast<T*>(this);
		HWND hWndCtrl = pT->GetDlgItem(nID);
		ATLASSERT(hWndCtrl);

		pT->OnOptionSelEndOk(CBN_SELENDOK, nID, hWndCtrl);
	}

	/*!
	** @brief   Handles the CBN_SELENDOK notification from the Combo Box control.
	** @param   [in] uNotifyCode the notification code.
	** @param   [in] nID the identifier of the control sending the message.
	** @param   [in] hWndCtrl a handle to the control sending the message.
	** @param   [in, out] bHandled set to FALSE if further processing is needed.
	** @returns The result of the message processing.
	** @remarks -
	*/
	LRESULT OnOptionSelEndOk(UINT uNotifyCode, int nID, HWND hWndCtrl, BOOL& bHandled)
	{
		UNREFERENCED_PARAMETER(bHandled);

		T* pT = static_cast<T*>(this);
		pT->OnOptionSelEndOk(uNotifyCode, nID, hWndCtrl);
		pT->SetModified();
		return LRESULT(0);
	}

	/*!
	** @brief   Handles the CBN_SELENDOK notification from the Combo Box control.
	** @param   [in] uNotifyCode the notification code.
	** @param   [in] nID the identifier of the control sending the message.
	** @param   [in] hWndCtrl a handle to the control sending the message.
	** @returns No return value.
	** @remarks -
	*/
	void OnOptionSelEndOk(UINT uNotifyCode, int nID, HWND hWndCtrl)
	{
		UNREFERENCED_PARAMETER(uNotifyCode);

		T* pT = static_cast<T*>(this);

		int iCurSel = CComboBox(hWndCtrl).GetCurSel();

		switch (nID)
		{
		case IDC_OPTION_CREATIONTIME:

			switch (iCurSel)
			{
			case T::CCreationTimeOptions::SetSpecifiedDateAndTime:
			case T::CCreationTimeOptions::SetSpecifiedTimeOfDay:
				pT->EnableDlgCtrl(IDC_DATETIME_CREATIONTIME, TRUE);
				break;
			default:
				pT->EnableDlgCtrl(IDC_DATETIME_CREATIONTIME, FALSE);
				break;
			}
			break;

		case IDC_OPTION_LASTACCESSTIME:

			switch (iCurSel)
			{
			case T::CLastAccessTimeOptions::SetSpecifiedDateAndTime:
			case T::CLastAccessTimeOptions::SetSpecifiedTimeOfDay:
				pT->EnableDlgCtrl(IDC_DATETIME_LASTACCESSTIME, TRUE);
				break;
			default:
				pT->EnableDlgCtrl(IDC_DATETIME_LASTACCESSTIME, FALSE);
				break;
			}
			break;

		case IDC_OPTION_LASTWRITETIME:

			switch (iCurSel)
			{
			case T::CLastWriteTimeOptions::SetSpecifiedDateAndTime:
			case T::CLastWriteTimeOptions::SetSpecifiedTimeOfDay:
				pT->EnableDlgCtrl(IDC_DATETIME_LASTWRITETIME, TRUE);
				break;
			default:
				pT->EnableDlgCtrl(IDC_DATETIME_LASTWRITETIME, FALSE);
				break;
			}
			break;
		};
	}

	/*!
	** @brief   Substitutes for a DTN_DATETIMECHANGE notification from the Date and Time Picker control.
	** @param   [in] nID the identifier of the control sending the message.
	** @returns No return value.
	** @remarks -
	*/
	void OnDateTimeChange(int nID)
	{
		T* pT = static_cast<T*>(this);
		HWND hWndCtrl = pT->GetDlgItem(nID);
		ATLASSERT(hWndCtrl);

		NMDATETIMECHANGE change;
		change.nmhdr.hwndFrom = hWndCtrl;
		change.nmhdr.idFrom = nID;
		change.nmhdr.code = DTN_DATETIMECHANGE;

		change.dwFlags = CDateTimePickerCtrl(hWndCtrl).GetSystemTime(&change.st);

		pT->OnDateTimeChange(nID, &change.nmhdr);
	}

	/*!
	** @brief   Handles the DTN_DATETIMECHANGE notification from the Date and Time Picker control.
	** @param   [in] nID the identifier of the control sending the message.
	** @param   [in] pnmh a pointer to information about the notification message.
	** @param   [in, out] bHandled set to FALSE if further processing is needed.
	** @returns The result of the message processing.
	** @remarks -
	*/
	LRESULT OnDateTimeChange(int nID, LPNMHDR pnmh, BOOL& bHandled)
	{
		UNREFERENCED_PARAMETER(bHandled);

		T* pT = static_cast<T*>(this);
		pT->OnDateTimeChange(nID, pnmh);
		pT->SetModified();
		return LRESULT(0);
	}

	/*!
	** @brief   Handles the DTN_DATETIMECHANGE notification from the Date and Time Picker control.
	** @param   [in] nID the identifier of the control sending the message.
	** @param   [in] pnmh a pointer to information about the notification message.
	** @returns No return value.
	** @remarks -
	*/
	void OnDateTimeChange(int nID, LPNMHDR pnmh)
	{
		LPNMDATETIMECHANGE lpNMDateTimeChange = reinterpret_cast<LPNMDATETIMECHANGE>(pnmh);

		DBG_UNREFERENCED_LOCAL_VARIABLE(lpNMDateTimeChange);

		T* pT = static_cast<T*>(this);

		switch (nID)
		{
		case IDC_DATETIME_CREATIONTIME:
		case IDC_DATETIME_LASTACCESSTIME:
		case IDC_DATETIME_LASTWRITETIME:

			DBG_UNREFERENCED_LOCAL_VARIABLE(pT);

			break;
		};
	}

	/*!
	** @brief   Called when the user clicks the Current Date and Time button.
	** @param   [in] uNotifyCode the notification code.
	** @param   [in] nID the identifier of the menu item, control, or accelerator.
	** @param   [in] hWndCtrl a handle to the control.
	** @param   [in, out] bHandled set to FALSE if further processing is needed.
	** @returns The result of the message processing.
	** @remarks -
	*/
	LRESULT OnCurrent(UINT uNotifyCode, int nID, HWND hWndCtrl, BOOL& bHandled)
	{
		UNREFERENCED_PARAMETER(bHandled);

		T* pT = static_cast<T*>(this);
		pT->OnCurrent(uNotifyCode, nID, hWndCtrl);
		return LRESULT(0);
	}

	/*!
	** @brief   Called when the user clicks the Current Date and Time button.
	** @param   [in] uNotifyCode the notification code.
	** @param   [in] nID the identifier of the menu item, control, or accelerator.
	** @param   [in] hWndCtrl a handle to the control.
	** @returns No return value.
	** @remarks -
	*/
	void OnCurrent(UINT uNotifyCode, int nID, HWND hWndCtrl)
	{
		UNREFERENCED_PARAMETER(uNotifyCode);
		UNREFERENCED_PARAMETER(nID);
		UNREFERENCED_PARAMETER(hWndCtrl);

		T* pT = static_cast<T*>(this);

		m_iCreationTimeOption   = T::CCreationTimeOptions::SetSpecifiedDateAndTime;
		m_iLastAccessTimeOption = T::CLastAccessTimeOptions::SetSpecifiedDateAndTime;
		m_iLastWriteTimeOption  = T::CLastWriteTimeOptions::SetSpecifiedDateAndTime;

		CFileTimeEx ftCurrentTime = CFileTimeEx::GetCurrentTime().ZeroMillisecond();

		m_ftCreationTime   = ftCurrentTime;
		m_ftLastAccessTime = ftCurrentTime;
		m_ftLastWriteTime  = ftCurrentTime;

		DoDataExchange(DDX_LOAD);

		pT->SetModified();
	}

	/*!
	** @brief   Called when the user clicks the Browse button.
	** @param   [in] uNotifyCode the notification code.
	** @param   [in] nID the identifier of the menu item, control, or accelerator.
	** @param   [in] hWndCtrl a handle to the control.
	** @param   [in, out] bHandled set to FALSE if further processing is needed.
	** @returns The result of the message processing.
	** @remarks -
	*/
	LRESULT OnBrowse(UINT uNotifyCode, int nID, HWND hWndCtrl, BOOL& bHandled)
	{
		UNREFERENCED_PARAMETER(bHandled);

		T* pT = static_cast<T*>(this);
		pT->OnBrowse(uNotifyCode, nID, hWndCtrl);
		return LRESULT(0);
	}

	/*!
	** @brief   Called when the user clicks the Browse button.
	** @param   [in] uNotifyCode the notification code.
	** @param   [in] nID the identifier of the menu item, control, or accelerator.
	** @param   [in] hWndCtrl a handle to the control.
	** @returns No return value.
	** @remarks -
	*/
	void OnBrowse(UINT uNotifyCode, int nID, HWND hWndCtrl)
	{
		UNREFERENCED_PARAMETER(uNotifyCode);
		UNREFERENCED_PARAMETER(nID);
		UNREFERENCED_PARAMETER(hWndCtrl);

		T* pT = static_cast<T*>(this);

		HRESULT hResult = S_OK;

		CShellFileOpenDialogEx wndFileDialog;
		if (wndFileDialog.DoModal() != IDOK)
			return;

		CFile file;
		hResult = wndFileDialog.GetFilePath(file);
		ATLASSERT(SUCCEEDED(hResult));

		CFileTimeEx ftCreationTime;
		CFileTimeEx ftLastAccessTime;
		CFileTimeEx ftLastWriteTime;

		if (file.GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime))
		{
			m_iCreationTimeOption   = T::CCreationTimeOptions::SetSpecifiedDateAndTime;
			m_iLastAccessTimeOption = T::CLastAccessTimeOptions::SetSpecifiedDateAndTime;
			m_iLastWriteTimeOption  = T::CLastWriteTimeOptions::SetSpecifiedDateAndTime;

			m_ftCreationTime   = ftCreationTime;
			m_ftLastAccessTime = ftLastAccessTime;
			m_ftLastWriteTime  = ftLastWriteTime;

			DoDataExchange(DDX_LOAD);

			pT->SetModified();
		}
		else
		{
			hResult = AtlHresultFromLastError();

			pT->Alert(hResult);
		}
	}
};

/*!
** @brief   The general handling options for file times.
** @remarks -
*/
template <class T>
class CGeneralOptions : public CHandlingOptions<T>
{
public:

	/*!
	** @brief   The handling options for the creation time.
	** @remarks -
	*/
	struct CCreationTimeOptions
	{
		enum
		{
			DoNotChange,
			SetSpecifiedDateAndTime,
			SetSpecifiedTimeOfDay,
			SetCurrentDateAndTime,
			SetTimeOfDayToMidnight,
			SetSecondToZero,
			SetSameAsModificationTime,
			SetSameAsAccessTime,
			SetSameAsEarliestChild,
			SetSameAsLatestChild,
		};
	};

	/*!
	** @brief   The handling options for the access time.
	** @remarks -
	*/
	struct CLastAccessTimeOptions
	{
		enum
		{
			DoNotChange,
			SetSpecifiedDateAndTime,
			SetSpecifiedTimeOfDay,
			SetCurrentDateAndTime,
			SetTimeOfDayToMidnight,
			SetSecondToZero,
			SetSameAsCreationTime,
			SetSameAsModificationTime,
			SetSameAsEarliestChild,
			SetSameAsLatestChild,
		};
	};

	/*!
	** @brief   The handling options for the modification time.
	** @remarks -
	*/
	struct CLastWriteTimeOptions
	{
		enum
		{
			DoNotChange,
			SetSpecifiedDateAndTime,
			SetSpecifiedTimeOfDay,
			SetCurrentDateAndTime,
			SetTimeOfDayToMidnight,
			SetSecondToZero,
			SetSameAsCreationTime,
			SetSameAsAccessTime,
			SetSameAsEarliestChild,
			SetSameAsLatestChild,
		};
	};

public:

	/*!
	** @brief   Initializes a new CGeneralOptions object.
	** @remarks -
	*/
	CGeneralOptions()
	{
		;
	}

	/*!
	** @copydoc CGeneralPage::OnInitDialog
	*/
	BOOL OnInitDialog(HWND hWndFocus, LPARAM lInitParam)
	{
		UNREFERENCED_PARAMETER(hWndFocus);
		UNREFERENCED_PARAMETER(lInitParam);

		T* pT = static_cast<T*>(this);

		if (pT->m_bDirectory)
		{
			;
		}
		else
		{
			CComboBox wndCreationTimeOption   = pT->GetDlgItem(IDC_OPTION_CREATIONTIME);
			CComboBox wndLastAccessTimeOption = pT->GetDlgItem(IDC_OPTION_LASTACCESSTIME);
			CComboBox wndLastWriteTimeOption  = pT->GetDlgItem(IDC_OPTION_LASTWRITETIME);

			for (int i = wndCreationTimeOption.GetCount(); i-- > T::CCreationTimeOptions::SetSameAsEarliestChild; )
				wndCreationTimeOption.DeleteString(i);
			for (int i = wndLastAccessTimeOption.GetCount(); i-- > T::CLastAccessTimeOptions::SetSameAsEarliestChild; )
				wndLastAccessTimeOption.DeleteString(i);
			for (int i = wndLastWriteTimeOption.GetCount(); i-- > T::CLastWriteTimeOptions::SetSameAsEarliestChild; )
				wndLastWriteTimeOption.DeleteString(i);
		}
		return TRUE;
	}
};

/*!
** @brief   The advanced handling options for file times.
** @remarks -
*/
template <class T>
class CAdvancedOptions : public CHandlingOptions<T>
{
public:

	/*!
	** @brief   The handling options for the creation time.
	** @remarks -
	*/
	struct CCreationTimeOptions
	{
		enum
		{
			DoNotChange,
			SetSpecifiedDateAndTime,
			SetSpecifiedTimeOfDay,
			SetCurrentDateAndTime,
			SetTimeOfDayToMidnight,
			SetSecondToZero,
			SetSameAsModificationTime,
			SetSameAsAccessTime,
		};
	};

	/*!
	** @brief   The handling options for the access time.
	** @remarks -
	*/
	struct CLastAccessTimeOptions
	{
		enum
		{
			DoNotChange,
			SetSpecifiedDateAndTime,
			SetSpecifiedTimeOfDay,
			SetCurrentDateAndTime,
			SetTimeOfDayToMidnight,
			SetSecondToZero,
			SetSameAsCreationTime,
			SetSameAsModificationTime,
		};
	};

	/*!
	** @brief   The handling options for the modification time.
	** @remarks -
	*/
	struct CLastWriteTimeOptions
	{
		enum
		{
			DoNotChange,
			SetSpecifiedDateAndTime,
			SetSpecifiedTimeOfDay,
			SetCurrentDateAndTime,
			SetTimeOfDayToMidnight,
			SetSecondToZero,
			SetSameAsCreationTime,
			SetSameAsAccessTime,
		};
	};

public:

	/*!
	** @brief   Initializes a new CAdvancedOptions object.
	** @remarks -
	*/
	CAdvancedOptions()
	{
		;
	}

	/*!
	** @copydoc CAdvancedPage::OnInitDialog
	*/
	BOOL OnInitDialog(HWND hWndFocus, LPARAM lInitParam)
	{
		UNREFERENCED_PARAMETER(hWndFocus);
		UNREFERENCED_PARAMETER(lInitParam);

		;

		return TRUE;
	}
};

/*!
** @brief   The base class of the context for handling options.
** @remarks -
*/
template <class T>
class CHandlingContext
{
public:

	typename T::CHandlingOptions& m_options; ///< The handling options for file times.

public:

	/*!
	** @brief   Constructs a new CHandlingContext object with the specified options.
	** @param   [in] options the handling options for file times.
	** @remarks -
	*/
	CHandlingContext(typename T::CHandlingOptions& options) : m_options(options)
	{
		;
	}

	/*!
	** @brief   Determines whether no operations are required.
	** @returns true if no operations, or false otherwise.
	** @remarks -
	*/
	bool HasNoOperations() const
	{
		if ((m_options.m_iCreationTimeOption   == T::CCreationTimeOptions::DoNotChange)
		 && (m_options.m_iLastAccessTimeOption == T::CLastAccessTimeOptions::DoNotChange)
		 && (m_options.m_iLastWriteTimeOption  == T::CLastWriteTimeOptions::DoNotChange))
		{
			return true;
		}
		return false;
	}

	/*!
	** @brief   Modifies the file times for a file or a directory.
	** @param   [in, out] ftCreationTime a FILETIME structure that contains the creation time.
	** @param   [in, out] ftLastAccessTime a FILETIME structure that contains the access time.
	** @param   [in, out] ftLastWriteTime a FILETIME structure that contains the modification time.
	** @returns No return value.
	** @remarks -
	*/
	void ModifyFileTime(FILETIME& ftCreationTime, FILETIME& ftLastAccessTime, FILETIME& ftLastWriteTime)
	{
		T::CHandlingContext* pThis = static_cast<T::CHandlingContext*>(this);

		// Phase 1: Read out file times

		CFileTimeEx accCreationTime   = ftCreationTime;
		CFileTimeEx accLastAccessTime = ftLastAccessTime;
		CFileTimeEx accLastWriteTime  = ftLastWriteTime;

		// Phase 2: Modify individual file times

		pThis->ModifyCreationTime(accCreationTime);
		pThis->ModifyLastAccessTime(accLastAccessTime);
		pThis->ModifyLastWriteTime(accLastWriteTime);

		// Phase 3: Write back file times

		switch (m_options.m_iCreationTimeOption)
		{
		case T::CCreationTimeOptions::SetSameAsAccessTime:
			ftCreationTime = accLastAccessTime;
			break;
		case T::CCreationTimeOptions::SetSameAsModificationTime:
			ftCreationTime = accLastWriteTime;
			break;
		default:
			ftCreationTime = accCreationTime;
			break;
		}
		switch (m_options.m_iLastAccessTimeOption)
		{
		case T::CLastAccessTimeOptions::SetSameAsCreationTime:
			ftLastAccessTime = accCreationTime;
			break;
		case T::CLastAccessTimeOptions::SetSameAsModificationTime:
			ftLastAccessTime = accLastWriteTime;
			break;
		default:
			ftLastAccessTime = accLastAccessTime;
			break;
		}
		switch (m_options.m_iLastWriteTimeOption)
		{
		case T::CLastWriteTimeOptions::SetSameAsCreationTime:
			ftLastWriteTime = accCreationTime;
			break;
		case T::CLastWriteTimeOptions::SetSameAsAccessTime:
			ftLastWriteTime = accLastAccessTime;
			break;
		default:
			ftLastWriteTime = accLastWriteTime;
			break;
		}
	}

protected:

	/*!
	** @brief   Modifies the creation time.
	** @param   [in, out] accCreationTime the creation time to be modified.
	** @returns No return value.
	** @remarks -
	*/
	void ModifyCreationTime(CFileTimeEx& accCreationTime)
	{
		switch (m_options.m_iCreationTimeOption)
		{
		case T::CCreationTimeOptions::DoNotChange:
			accCreationTime;
			break;
		case T::CCreationTimeOptions::SetSpecifiedDateAndTime:
			accCreationTime = m_options.m_ftCreationTime;
			break;
		case T::CCreationTimeOptions::SetSpecifiedTimeOfDay:
			accCreationTime = accCreationTime.GetTzDate() + m_options.m_ftCreationTime.GetTzTime();
			break;
		case T::CCreationTimeOptions::SetCurrentDateAndTime:
			accCreationTime = m_options.m_ftCurrentTime;
			break;
		case T::CCreationTimeOptions::SetTimeOfDayToMidnight:
			accCreationTime = accCreationTime.GetTzDate();
			break;
		case T::CCreationTimeOptions::SetSecondToZero:
			accCreationTime = accCreationTime.ZeroSecond();
			break;
		}
	}

	/*!
	** @brief   Modifies the access time.
	** @param   [in, out] accLastAccessTime the access time to be modified.
	** @returns No return value.
	** @remarks -
	*/
	void ModifyLastAccessTime(CFileTimeEx& accLastAccessTime)
	{
		switch (m_options.m_iLastAccessTimeOption)
		{
		case T::CLastAccessTimeOptions::DoNotChange:
			accLastAccessTime;
			break;
		case T::CLastAccessTimeOptions::SetSpecifiedDateAndTime:
			accLastAccessTime = m_options.m_ftLastAccessTime;
			break;
		case T::CLastAccessTimeOptions::SetSpecifiedTimeOfDay:
			accLastAccessTime = accLastAccessTime.GetTzDate() + m_options.m_ftLastAccessTime.GetTzTime();
			break;
		case T::CLastAccessTimeOptions::SetCurrentDateAndTime:
			accLastAccessTime = m_options.m_ftCurrentTime;
			break;
		case T::CLastAccessTimeOptions::SetTimeOfDayToMidnight:
			accLastAccessTime = accLastAccessTime.GetTzDate();
			break;
		case T::CLastAccessTimeOptions::SetSecondToZero:
			accLastAccessTime = accLastAccessTime.ZeroSecond();
			break;
		}
	}

	/*!
	** @brief   Modifies the modification time.
	** @param   [in, out] accLastWriteTime the modification time to be modified.
	** @returns No return value.
	** @remarks -
	*/
	void ModifyLastWriteTime(CFileTimeEx& accLastWriteTime)
	{
		switch (m_options.m_iLastWriteTimeOption)
		{
		case T::CLastWriteTimeOptions::DoNotChange:
			accLastWriteTime;
			break;
		case T::CLastWriteTimeOptions::SetSpecifiedDateAndTime:
			accLastWriteTime = m_options.m_ftLastWriteTime;
			break;
		case T::CLastWriteTimeOptions::SetSpecifiedTimeOfDay:
			accLastWriteTime = accLastWriteTime.GetTzDate() + m_options.m_ftLastWriteTime.GetTzTime();
			break;
		case T::CLastWriteTimeOptions::SetCurrentDateAndTime:
			accLastWriteTime = m_options.m_ftCurrentTime;
			break;
		case T::CLastWriteTimeOptions::SetTimeOfDayToMidnight:
			accLastWriteTime = accLastWriteTime.GetTzDate();
			break;
		case T::CLastWriteTimeOptions::SetSecondToZero:
			accLastWriteTime = accLastWriteTime.ZeroSecond();
			break;
		}
	}
};

/*!
** @brief   The context for the general handling options.
** @remarks -
*/
template <class T>
class CGeneralContext : public CHandlingContext<T>
{
private:

	friend class CHandlingContext<T>;

public:

	CFileTimeEx m_ftEarliestCreationTime;   ///< The earliest creation time of child files and subdirectories.
	CFileTimeEx m_ftEarliestLastAccessTime; ///< The earliest access time of child files and subdirectories.
	CFileTimeEx m_ftEarliestLastWriteTime;  ///< The earliest modification time of child files and subdirectories,

	CFileTimeEx m_ftLatestCreationTime;     ///< The latest creation time of child files and subdirectories.
	CFileTimeEx m_ftLatestLastAccessTime;   ///< The latest access time of child files and subdirectories.
	CFileTimeEx m_ftLatestLastWriteTime;    ///< The latest modification time of child files and subdirectories,

public:

	/*!
	** @brief   Constructs a new CGeneralContext object with the specified options.
	** @param   [in] options the handling options for file times.
	** @remarks -
	*/
	CGeneralContext(typename T::CHandlingOptions& options) : CHandlingContext<T>(options)
	{
		m_ftEarliestCreationTime   = ULONGLONG(MAXLONGLONG), m_ftLatestCreationTime   = 0;
		m_ftEarliestLastAccessTime = ULONGLONG(MAXLONGLONG), m_ftLatestLastAccessTime = 0;
		m_ftEarliestLastWriteTime  = ULONGLONG(MAXLONGLONG), m_ftLatestLastWriteTime  = 0;
	}

protected:

	/*!
	** @copydoc CHandlingContext::ModifyCreationTime
	*/
	void ModifyCreationTime(CFileTimeEx& accCreationTime)
	{
		switch (m_options.m_iCreationTimeOption)
		{
		case T::CCreationTimeOptions::SetSameAsEarliestChild:
			accCreationTime = m_ftEarliestCreationTime;
			break;
		case T::CCreationTimeOptions::SetSameAsLatestChild:
			accCreationTime = m_ftLatestCreationTime;
			break;
		default:
			CHandlingContext<T>::ModifyCreationTime(accCreationTime);
			break;
		}
	}

	/*!
	** @copydoc CHandlingContext::ModifyLastAccessTime
	*/
	void ModifyLastAccessTime(CFileTimeEx& accLastAccessTime)
	{
		switch (m_options.m_iLastAccessTimeOption)
		{
		case T::CLastAccessTimeOptions::SetSameAsEarliestChild:
			accLastAccessTime = m_ftEarliestLastAccessTime;
			break;
		case T::CLastAccessTimeOptions::SetSameAsLatestChild:
			accLastAccessTime = m_ftLatestLastAccessTime;
			break;
		default:
			CHandlingContext<T>::ModifyLastAccessTime(accLastAccessTime);
			break;
		}
	}

	/*!
	** @copydoc CHandlingContext::ModifyLastWriteTime
	*/
	void ModifyLastWriteTime(CFileTimeEx& accLastWriteTime)
	{
		switch (m_options.m_iLastWriteTimeOption)
		{
		case T::CLastWriteTimeOptions::SetSameAsEarliestChild:
			accLastWriteTime = m_ftEarliestLastWriteTime;
			break;
		case T::CLastWriteTimeOptions::SetSameAsLatestChild:
			accLastWriteTime = m_ftLatestLastWriteTime;
			break;
		default:
			CHandlingContext<T>::ModifyLastWriteTime(accLastWriteTime);
			break;
		}
	}
};

/*!
** @brief   The context for the advanced handling options.
** @remarks -
*/
template <class T>
class CAdvancedContext : public CHandlingContext<T>
{
private:

	friend class CHandlingContext<T>;

public:

	/*!
	** @brief   Constructs a new CAdvancedContext object with the specified options.
	** @param   [in] options the handling options for file times.
	** @remarks -
	*/
	CAdvancedContext(typename T::CHandlingOptions& options) : CHandlingContext<T>(options)
	{
		;
	}

protected:

	/*!
	** @copydoc CHandlingContext::ModifyCreationTime
	*/
	void ModifyCreationTime(CFileTimeEx& accCreationTime)
	{
		#pragma warning(push)
		#pragma warning(disable: 4065)

		switch (m_options.m_iCreationTimeOption)
		{
		default:
			CHandlingContext<T>::ModifyCreationTime(accCreationTime);
			break;
		}
		#pragma warning(pop)
	}

	/*!
	** @copydoc CHandlingContext::ModifyLastAccessTime
	*/
	void ModifyLastAccessTime(CFileTimeEx& accLastAccessTime)
	{
		#pragma warning(push)
		#pragma warning(disable: 4065)

		switch (m_options.m_iLastAccessTimeOption)
		{
		default:
			CHandlingContext<T>::ModifyLastAccessTime(accLastAccessTime);
			break;
		}
		#pragma warning(pop)
	}

	/*!
	** @copydoc CHandlingContext::ModifyLastWriteTime
	*/
	void ModifyLastWriteTime(CFileTimeEx& accLastWriteTime)
	{
		#pragma warning(push)
		#pragma warning(disable: 4065)

		switch (m_options.m_iLastWriteTimeOption)
		{
		default:
			CHandlingContext<T>::ModifyLastWriteTime(accLastWriteTime);
			break;
		}
		#pragma warning(pop)
	}
};
