/*!@file
** @brief   The entry point of the program.
** @remarks This file is the single compilation unit of the program.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
**
** @copyright (C) 2014-2018 Ueta System Design Studio\n
** This file is licensed under the terms of the MIT License.\n
** See <https://opensource.org/licenses/MIT>.
*/
#include "stdafx.h"
#include "resource.h"
#include "dialog.h"

/*!
** @brief   The global instance of CAppModule.
** @remarks -
*/
CAppModule _Module;

/*!
** @brief   The property page manipulator.
** @remarks -
*/
class CManipulator : public CPropertyPageWindow
{
public:

	/*!
	** @brief   Constructs a new CManipulator object to manipulate the specified property page.
	** @param   [in] hPageDlg a handle to the property page to be manipulated.
	** @remarks -
	*/
	CManipulator(HWND hPageDlg = NULL) : CPropertyPageWindow(hPageDlg)
	{
		;
	}

	/*!
	** @brief   Pauses the execution of the current thread for the specified time.
	** @param   [in] dwMilliseconds the time, in milliseconds, for which to suspend execution.
	** @returns No return value.
	** @remarks -
	*/
	static void Pause(DWORD dwMilliseconds = 1000)
	{
		::Sleep(dwMilliseconds);
	}

	/*!
	** @brief   Sets the currently selected item in a Combo Box control.
	** @param   [in] nCtrlID the identifier of the control.
	** @param   [in] nSelect the zero-based index of the item to select.
	** @returns the index of the item selected if successful, or CB_ERR otherwise.
	** @remarks -
	*/
	int SetCurSel(int nCtrlID, int nSelect)
	{
		CComboBox wndComboBox = GetDlgItem(nCtrlID);

		int nSelected = wndComboBox.SetCurSel(nSelect);

		if (nSelected != CB_ERR)
		{
			SendMessage(WM_COMMAND, MAKEWPARAM(nCtrlID, CBN_SELENDOK), LPARAM(HWND(wndComboBox)));
		}
		return nSelected;
	}

	/*!
	** @brief   Sets a date and time to a Date and Time Picker control.
	** @param   [in] nCtrlID the identifier of the control.
	** @param   [in, out] ft a FILETIME structure that contains the date and time.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL SetFileTime(int nCtrlID, FILETIME& ft)
	{
		CDateTimePickerCtrlEx wndDateTimePickerCtrlEx = GetDlgItem(nCtrlID);

		if (wndDateTimePickerCtrlEx.SetFileTime(GDT_VALID, &ft))
		{
			if (wndDateTimePickerCtrlEx.GetFileTime(&ft) == GDT_VALID)
			{
				return TRUE;
			}
		}
		return FALSE;
	}
};

namespace
{
	/*!
	** @brief   Creates a new directory.
	** @param   [in] lpPathName the path of the directory to be created.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL CreateDirectory(LPCTSTR lpPathName)
	{
		return ::CreateDirectory(lpPathName, NULL);
	}

	/*!
	** @brief   Creates a new file.
	** @param   [in] lpFileName the path of the file to be created.
	** @param   [in] dwFlagsAndAttributes the file attributes and flags.
	** @returns TRUE if successful, or FALSE otherwise.
	** @remarks -
	*/
	BOOL CreateFile(LPCTSTR lpFileName, DWORD dwFlagsAndAttributes = FILE_ATTRIBUTE_NORMAL)
	{
		HANDLE hFile = ::CreateFile(lpFileName, GENERIC_WRITE, 0, NULL, CREATE_NEW, dwFlagsAndAttributes, NULL);

		if (hFile != INVALID_HANDLE_VALUE)
		{
			::CloseHandle(hFile);

			return TRUE;
		}
		return FALSE;
	}
};

/*!
** @brief   Implements the entry point for the application.
** @param   [in] hInstance a handle to the current instance of the application.
** @param   [in] hPrevInstance a legacy from 16-bit Windows, now always zero.
** @param   [in] lpCmdLine the command-line for the application.
** @param   [in] nCmdShow specifies how the window is to be shown.
** @returns A standard HRESULT value as the exit value.
** @remarks -
*/
extern "C" int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	UNREFERENCED_PARAMETER(hInstance);
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);
	UNREFERENCED_PARAMETER(nCmdShow);

	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);

	HRESULT hResult = S_OK;

	hResult = ::CoInitialize(NULL);
	ATLASSERT(SUCCEEDED(hResult));
	::DefWindowProc(NULL, WM_NULL, 0, 0L);
	AtlInitCommonControls(ICC_WIN95_CLASSES | ICC_DATE_CLASSES);
	ATLASSERT(!CAtlBaseModule::m_bInitFailed);
	hResult = _Module.Init(NULL, hInstance);
	ATLASSERT(SUCCEEDED(hResult));

	CMessageLoop theMessageLoop;

	ATLVERIFY(_Module.AddMessageLoop(&theMessageLoop));

	#if 1 // Setup
	{
		ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk")));

		ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk\\directory1")));
		ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk\\directory1\\subdirectory1")));
	//	ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk\\directory1\\subdirectory2")));
		ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk\\directory1\\subdirectory3")));
		ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk\\directory2")));
		ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk\\directory2\\subdirectory1")));
		ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk\\directory2\\subdirectory2")));
		ATLVERIFY(CreateDirectory(TEXT(".\\target\\junk\\directory2\\subdirectory3")));

		ATLVERIFY(CreateSymbolicLink(
				TEXT(".\\target\\junk\\directory1\\subdirectory2"),
				TEXT(             "..\\directory2\\subdirectory2"), SYMBOLIC_LINK_FLAG_DIRECTORY));

		ATLVERIFY(SetFileAttributes(
				TEXT(".\\target\\junk\\directory1\\subdirectory3"), FILE_ATTRIBUTE_HIDDEN));
		ATLVERIFY(SetFileAttributes(
				TEXT(".\\target\\junk\\directory2\\subdirectory3"), FILE_ATTRIBUTE_HIDDEN));

		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\file1")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\file2")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\file3")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\subdirectory1\\file1")));
	//	ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\subdirectory1\\file2")));
	//	ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\subdirectory2\\file1")));
	//	ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\subdirectory2\\file2")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\subdirectory3\\file1")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory1\\subdirectory3\\file2")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\file1")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\file2")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\file3")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\subdirectory1\\file1")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\subdirectory1\\file2")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\subdirectory2\\file1")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\subdirectory2\\file2")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\subdirectory3\\file1")));
		ATLVERIFY(CreateFile(TEXT(".\\target\\junk\\directory2\\subdirectory3\\file2")));

		ATLVERIFY(CreateSymbolicLink(
				TEXT(".\\target\\junk\\directory1\\subdirectory1\\file2"),
				TEXT(         "..\\..\\directory2\\subdirectory1\\file2"), 0));

		ATLVERIFY(SetFileAttributes(
				TEXT(".\\target\\junk\\directory1\\file3"), FILE_ATTRIBUTE_HIDDEN));
		ATLVERIFY(SetFileAttributes(
				TEXT(".\\target\\junk\\directory2\\file3"), FILE_ATTRIBUTE_HIDDEN));
	}
	#endif

	#if 1 // Test 1: Directory
	{
		CFile files[9];

		files[0] = TEXT(".\\target\\junk\\directory1");
		files[1] = TEXT(".\\target\\junk\\directory1\\file1");
		files[2] = TEXT(".\\target\\junk\\directory1\\subdirectory2");
		files[3] = TEXT(".\\target\\junk\\directory1\\subdirectory1\\file2");
		files[4] = TEXT(".\\target\\junk\\directory2\\subdirectory2");
		files[5] = TEXT(".\\target\\junk\\directory2\\subdirectory1\\file2");
		files[6] = TEXT(".\\target\\junk\\directory1\\file3");
		files[7] = TEXT(".\\target\\junk\\directory1\\subdirectory3");
		files[8] = TEXT(".\\target\\junk\\directory1\\subdirectory3\\file1");

		CFileTimeEx ftCreationTime[_countof(files)],   ftExpectedCreationTime[_countof(files)];
		CFileTimeEx ftLastAccessTime[_countof(files)], ftExpectedLastAccessTime[_countof(files)];
		CFileTimeEx ftLastWriteTime[_countof(files)],  ftExpectedLastWriteTime[_countof(files)];

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ftExpectedCreationTime[i]   = ftCreationTime[i];
			ftExpectedLastAccessTime[i] = ftLastAccessTime[i];
			ftExpectedLastWriteTime[i]  = ftLastWriteTime[i];
		}

		CDialog theDialog(files[0]);
		ATLVERIFY(theDialog.Create(HWND_DESKTOP));
		ATLASSERT(theDialog.IsWindowVisible());
		ATLASSERT(theDialog.GetPageCount() == 2);

		CManipulator general  = theDialog.IndexToHwnd(0);
		CManipulator advanced = theDialog.IndexToHwnd(1);

		CManipulator::Pause();

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   1) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ftExpectedCreationTime[0] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_CREATIONTIME, ftExpectedCreationTime[0]));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 1) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ftExpectedLastAccessTime[0] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTACCESSTIME, ftExpectedLastAccessTime[0]));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  1) != CB_ERR);

		ftExpectedLastWriteTime[0] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTWRITETIME, ftExpectedLastWriteTime[0]));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(::CheckDlgButton(general, IDC_RECURSIVE, BST_CHECKED));

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   1) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 1) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  1) != CB_ERR);

		ftExpectedCreationTime[0]   = CFileTimeEx::GetCurrentTime();
		ftExpectedLastAccessTime[0] = CFileTimeEx::GetCurrentTime();
		ftExpectedLastWriteTime[0]  = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_CREATIONTIME,   ftExpectedCreationTime[0]));
		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTACCESSTIME, ftExpectedLastAccessTime[0]));
		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTWRITETIME,  ftExpectedLastWriteTime[0]));

		ftExpectedCreationTime[2]   = ftExpectedCreationTime[0];
		ftExpectedLastAccessTime[2] = ftExpectedLastAccessTime[0];
		ftExpectedLastWriteTime[2]  = ftExpectedLastWriteTime[0];

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ATLVERIFY(theDialog.UpdateWindow());

		CManipulator::Pause();

		ATLVERIFY(theDialog.SetActivePage(1));
		ATLVERIFY(theDialog.UpdateWindow());

		CManipulator::Pause();

		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_CREATIONTIME,   1) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ftExpectedCreationTime[1] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_CREATIONTIME, ftExpectedCreationTime[1]));

		ftExpectedCreationTime[3] = ftExpectedCreationTime[1];

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTACCESSTIME, 1) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ftExpectedLastAccessTime[1] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_LASTACCESSTIME, ftExpectedLastAccessTime[1]));

		ftExpectedLastAccessTime[3] = ftExpectedLastAccessTime[1];

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTWRITETIME,  1) != CB_ERR);

		ftExpectedLastWriteTime[1] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_LASTWRITETIME, ftExpectedLastWriteTime[1]));

		ftExpectedLastWriteTime[3] = ftExpectedLastWriteTime[1];

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(advanced.SetDlgItemText(IDC_FILESPEC, TEXT("*1;*2")));

		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_CREATIONTIME,   1) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTACCESSTIME, 1) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTWRITETIME,  1) != CB_ERR);

		ftExpectedCreationTime[1]   = CFileTimeEx::GetCurrentTime();
		ftExpectedLastAccessTime[1] = CFileTimeEx::GetCurrentTime();
		ftExpectedLastWriteTime[1]  = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_CREATIONTIME,   ftExpectedCreationTime[1]));
		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_LASTACCESSTIME, ftExpectedLastAccessTime[1]));
		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_LASTWRITETIME,  ftExpectedLastWriteTime[1]));

		ftExpectedCreationTime[3]   = ftExpectedCreationTime[1];
		ftExpectedLastAccessTime[3] = ftExpectedLastAccessTime[1];
		ftExpectedLastWriteTime[3]  = ftExpectedLastWriteTime[1];

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(advanced.SetDlgItemText(IDC_FILESPEC, TEXT("*3")));

		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_CREATIONTIME,   1) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTACCESSTIME, 1) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTWRITETIME,  1) != CB_ERR);

		ftCreationTime[1]   = CFileTimeEx::GetCurrentTime();
		ftLastAccessTime[1] = CFileTimeEx::GetCurrentTime();
		ftLastWriteTime[1]  = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_CREATIONTIME,   ftCreationTime[1]));
		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_LASTACCESSTIME, ftLastAccessTime[1]));
		ATLVERIFY(advanced.SetFileTime(IDC_DATETIME_LASTWRITETIME,  ftLastWriteTime[1]));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(advanced.SetDlgItemText(IDC_FILESPEC, NULL));

		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(advanced.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ATLVERIFY(theDialog.UpdateWindow());

		CManipulator::Pause();

		ATLVERIFY(theDialog.DestroyWindow());
	}
	#endif

	#if 1 // Test 2: File
	{
		CFile files[2];

		files[0] = TEXT(".\\target\\junk\\directory1\\subdirectory1\\file2");
		files[1] = TEXT(".\\target\\junk\\directory2\\subdirectory1\\file2");

		CFileTimeEx ftCreationTime[_countof(files)],   ftExpectedCreationTime[_countof(files)];
		CFileTimeEx ftLastAccessTime[_countof(files)], ftExpectedLastAccessTime[_countof(files)];
		CFileTimeEx ftLastWriteTime[_countof(files)],  ftExpectedLastWriteTime[_countof(files)];

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ftExpectedCreationTime[i]   = ftCreationTime[i];
			ftExpectedLastAccessTime[i] = ftLastAccessTime[i];
			ftExpectedLastWriteTime[i]  = ftLastWriteTime[i];
		}

		CDialog theDialog(files[0]);
		ATLVERIFY(theDialog.Create(HWND_DESKTOP));
		ATLASSERT(theDialog.IsWindowVisible());
		ATLASSERT(theDialog.GetPageCount() == 1);

		CManipulator general = theDialog.IndexToHwnd(0);

		CManipulator::Pause();

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   1) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ftExpectedCreationTime[0] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_CREATIONTIME, ftExpectedCreationTime[0]));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 1) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ftExpectedLastAccessTime[0] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTACCESSTIME, ftExpectedLastAccessTime[0]));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  1) != CB_ERR);

		ftExpectedLastWriteTime[0] = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTWRITETIME, ftExpectedLastWriteTime[0]));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		for (int i = 0; i < _countof(files); ++i)
		{
			ATLVERIFY(files[i].GetFileTime(&ftCreationTime[i], &ftLastAccessTime[i], &ftLastWriteTime[i]));

			ATLASSERT(ftCreationTime[i]   == ftExpectedCreationTime[i]);
			ATLASSERT(ftLastAccessTime[i] == ftExpectedLastAccessTime[i]);
			ATLASSERT(ftLastWriteTime[i]  == ftExpectedLastWriteTime[i]);
		}
		CManipulator::Pause();

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ATLVERIFY(theDialog.UpdateWindow());

		CManipulator::Pause();

		ATLVERIFY(theDialog.DestroyWindow());
	}
	#endif

	#if 1 // Test 3: Options
	{
		CFile files[3];

		files[0] = TEXT(".\\target\\junk\\directory1");
		files[1] = TEXT(".\\target\\junk\\directory1\\file1");
		files[2] = TEXT(".\\target\\junk\\directory1\\file2");

		CFileTimeEx ftCreationTime,   ftExpectedCreationTime;
		CFileTimeEx ftLastAccessTime, ftExpectedLastAccessTime;
		CFileTimeEx ftLastWriteTime,  ftExpectedLastWriteTime;
		CFileTimeEx ftCurrentTime;

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ftExpectedCreationTime   = ftCreationTime;
		ftExpectedLastAccessTime = ftLastAccessTime;
		ftExpectedLastWriteTime  = ftLastWriteTime;

		CDialog theDialog(files[0]);
		ATLVERIFY(theDialog.Create(HWND_DESKTOP));
		ATLASSERT(theDialog.IsWindowVisible());
		ATLASSERT(theDialog.GetPageCount() == 2);

		CManipulator general = theDialog.IndexToHwnd(0);

		CManipulator::Pause();

		#if 1 // Option: Set the specified time of day

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   2) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 2) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  2) != CB_ERR);

		ftCurrentTime = CFileTimeEx::GetCurrentTime();

		ftCreationTime   = ftCurrentTime - CFileTimeSpan(CFileTime::Second * 2);
		ftLastAccessTime = ftCurrentTime - CFileTimeSpan(CFileTime::Second * 1);
		ftLastWriteTime  = ftCurrentTime - CFileTimeSpan(CFileTime::Second * 0);

		if (ftCreationTime == ftCreationTime.GetTzDate())
			ftCreationTime -= CFileTimeSpan(CFileTime::Second * 1);
		if (ftLastAccessTime == ftLastAccessTime.GetTzDate())
			ftLastAccessTime -= CFileTimeSpan(CFileTime::Second * 2);
		if (ftLastWriteTime == ftLastWriteTime.GetTzDate())
			ftLastWriteTime -= CFileTimeSpan(CFileTime::Second * 3);

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_CREATIONTIME,   ftCreationTime));
		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTACCESSTIME, ftLastAccessTime));
		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTWRITETIME,  ftLastWriteTime));

		ftCreationTime   -= CFileTimeSpan(CFileTime::Day * 3);
		ftLastAccessTime -= CFileTimeSpan(CFileTime::Day * 2);
		ftLastWriteTime  -= CFileTimeSpan(CFileTime::Day * 1);

		ftExpectedCreationTime   = ftCreationTime;
		ftExpectedLastAccessTime = ftLastAccessTime;
		ftExpectedLastWriteTime  = ftLastWriteTime;

		ftCreationTime   = ftCreationTime.GetTzDate();
		ftLastAccessTime = ftLastAccessTime.GetTzDate();
		ftLastWriteTime  = ftLastWriteTime.GetTzDate();

		ATLVERIFY(files[0].SetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		#if 1 // Option: Set the current date and time

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   3) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 3) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  3) != CB_ERR);

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ftExpectedCreationTime   = theDialog.m_general.m_ftCurrentTime;
		ftExpectedLastAccessTime = theDialog.m_general.m_ftCurrentTime;
		ftExpectedLastWriteTime  = theDialog.m_general.m_ftCurrentTime;

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		#if 1 // Option: Set the time of day to midnight

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   4) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 4) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  4) != CB_ERR);

		ftCurrentTime = CFileTimeEx::GetCurrentTime();
		if (ftCurrentTime == ftCurrentTime.GetTzDate())
			ftCurrentTime += CFileTimeSpan(CFileTime::Hour * 12);

		ftCreationTime   = ftCurrentTime - CFileTimeSpan(CFileTime::Day * 3);
		ftLastAccessTime = ftCurrentTime - CFileTimeSpan(CFileTime::Day * 2);
		ftLastWriteTime  = ftCurrentTime - CFileTimeSpan(CFileTime::Day * 1);

		ftExpectedCreationTime   = ftCreationTime.GetTzDate();
		ftExpectedLastAccessTime = ftLastAccessTime.GetTzDate();
		ftExpectedLastWriteTime  = ftLastWriteTime.GetTzDate();

		ATLVERIFY(files[0].SetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		#if 1 // Option: Set the second to zero

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   5) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 5) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  5) != CB_ERR);

		ftCurrentTime = CFileTimeEx::GetCurrentTime();
		if (ftCurrentTime == ftCurrentTime.ZeroSecond())
			ftCurrentTime += CFileTimeSpan(CFileTime::Second * 30);

		ftCreationTime   = ftCurrentTime - CFileTimeSpan(CFileTime::Day * 3);
		ftLastAccessTime = ftCurrentTime - CFileTimeSpan(CFileTime::Day * 2);
		ftLastWriteTime  = ftCurrentTime - CFileTimeSpan(CFileTime::Day * 1);

		ftExpectedCreationTime   = ftCreationTime.ZeroSecond();
		ftExpectedLastAccessTime = ftLastAccessTime.ZeroSecond();
		ftExpectedLastWriteTime  = ftLastWriteTime.ZeroSecond();

		ATLVERIFY(files[0].SetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		#if 1 // Option: Set the same as the creation time

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   1) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 6) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  6) != CB_ERR);

		ftCurrentTime = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_CREATIONTIME, ftCurrentTime));

		ftExpectedCreationTime   = ftCurrentTime;
		ftExpectedLastAccessTime = ftCurrentTime;
		ftExpectedLastWriteTime  = ftCurrentTime;

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		#if 1 // Option: Set the same as the modification time

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   6) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 7) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  1) != CB_ERR);

		ftCurrentTime = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTWRITETIME, ftCurrentTime));

		ftExpectedCreationTime   = ftCurrentTime;
		ftExpectedLastAccessTime = ftCurrentTime;
		ftExpectedLastWriteTime  = ftCurrentTime;

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		#if 1 // Option: Set the same as the access time

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   7) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 1) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  7) != CB_ERR);

		ftCurrentTime = CFileTimeEx::GetCurrentTime();

		ATLVERIFY(general.SetFileTime(IDC_DATETIME_LASTACCESSTIME, ftCurrentTime));

		ftExpectedCreationTime   = ftCurrentTime;
		ftExpectedLastAccessTime = ftCurrentTime;
		ftExpectedLastWriteTime  = ftCurrentTime;

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		#if 1 // Option: Set the same as the earliest file/subfolder

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   8) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 8) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  8) != CB_ERR);

		ftCurrentTime = CFileTimeEx::GetCurrentTime();

		ftCreationTime   = ftCurrentTime + CFileTimeSpan(CFileTime::Hour * 1);
		ftLastAccessTime = ftCurrentTime + CFileTimeSpan(CFileTime::Hour * 2);
		ftLastWriteTime  = ftCurrentTime + CFileTimeSpan(CFileTime::Hour * 3);

		ATLVERIFY(files[2].SetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ftCreationTime   = ftCurrentTime - CFileTimeSpan(CFileTime::Hour * 3);
		ftLastAccessTime = ftCurrentTime - CFileTimeSpan(CFileTime::Hour * 2);
		ftLastWriteTime  = ftCurrentTime - CFileTimeSpan(CFileTime::Hour * 1);

		ftExpectedCreationTime   = ftCreationTime;
		ftExpectedLastAccessTime = ftLastAccessTime;
		ftExpectedLastWriteTime  = ftLastWriteTime;

		ATLVERIFY(files[1].SetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		#if 1 // Option: Set the same as the latest file/subfolder

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   9) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 9) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  9) != CB_ERR);

		ftCurrentTime = CFileTimeEx::GetCurrentTime();

		ftCreationTime   = ftCurrentTime + CFileTimeSpan(CFileTime::Hour * 1);
		ftLastAccessTime = ftCurrentTime + CFileTimeSpan(CFileTime::Hour * 2);
		ftLastWriteTime  = ftCurrentTime + CFileTimeSpan(CFileTime::Hour * 3);

		ftExpectedCreationTime   = ftCreationTime;
		ftExpectedLastAccessTime = ftLastAccessTime;
		ftExpectedLastWriteTime  = ftLastWriteTime;

		ATLVERIFY(files[2].SetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ftCreationTime   = ftCurrentTime - CFileTimeSpan(CFileTime::Hour * 3);
		ftLastAccessTime = ftCurrentTime - CFileTimeSpan(CFileTime::Hour * 2);
		ftLastWriteTime  = ftCurrentTime - CFileTimeSpan(CFileTime::Hour * 1);

		ATLVERIFY(files[1].SetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLVERIFY(theDialog.Apply());
		ATLVERIFY(theDialog.UpdateWindow());

		ATLVERIFY(files[0].GetFileTime(&ftCreationTime, &ftLastAccessTime, &ftLastWriteTime));

		ATLASSERT(ftCreationTime   == ftExpectedCreationTime);
		ATLASSERT(ftLastAccessTime == ftExpectedLastAccessTime);
		ATLASSERT(ftLastWriteTime  == ftExpectedLastWriteTime);

		CManipulator::Pause();

		#endif

		ATLVERIFY(general.SetCurSel(IDC_OPTION_CREATIONTIME,   0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTACCESSTIME, 0) != CB_ERR);
		ATLVERIFY(general.SetCurSel(IDC_OPTION_LASTWRITETIME,  0) != CB_ERR);

		ATLVERIFY(theDialog.UpdateWindow());

		CManipulator::Pause();

		ATLVERIFY(theDialog.DestroyWindow());
	}
	#endif

	#if 1 // Teardown
	{
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\subdirectory1\\file1")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\subdirectory1\\file2")));
	//	ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\subdirectory2\\file1")));
	//	ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\subdirectory2\\file2")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\subdirectory3\\file1")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\subdirectory3\\file2")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\file1")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\file2")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory1\\file3")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\subdirectory1\\file1")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\subdirectory1\\file2")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\subdirectory2\\file1")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\subdirectory2\\file2")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\subdirectory3\\file1")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\subdirectory3\\file2")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\file1")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\file2")));
		ATLVERIFY(DeleteFile(TEXT(".\\target\\junk\\directory2\\file3")));

		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk\\directory1\\subdirectory1")));
		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk\\directory1\\subdirectory2")));
		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk\\directory1\\subdirectory3")));
		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk\\directory1")));
		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk\\directory2\\subdirectory1")));
		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk\\directory2\\subdirectory2")));
		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk\\directory2\\subdirectory3")));
		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk\\directory2")));

		ATLVERIFY(RemoveDirectory(TEXT(".\\target\\junk")));
	}
	#endif

	ATLVERIFY(_Module.RemoveMessageLoop());

	_Module.Term();
	::CoUninitialize();
	return int(hResult);
}
